<?php

namespace GrowthExperiments\NewcomerTasks\TaskType;

use GrowthExperiments\NewcomerTasks\RecommendationProvider;
use GrowthExperiments\NewcomerTasks\RecommendationSubmissionHandler;

/**
 * TaskTypeHandler subclass for structured tasks. Instead of freeform editing, structured tasks
 * offer the user to apply a recommendation (typically generated by some machine learning system)
 * to the article.
 * @see https://www.mediawiki.org/wiki/Growth/Personalized_first_day/Structured_tasks
 */
abstract class StructuredTaskTypeHandler extends TaskTypeHandler {

	/**
	 * Returns the provider which can be used to fetch a recommendation for a given page.
	 * @return RecommendationProvider
	 */
	abstract public function getRecommendationProvider(): RecommendationProvider;

	/**
	 * Returns the handler which can be used to process a user submission (such as a user
	 * accepting or rejecting a recommendation).
	 * @return RecommendationSubmissionHandler
	 */
	abstract public function getSubmissionHandler(): RecommendationSubmissionHandler;

}
